/*
 * time.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "time.h"
#include <DS1307.h>
#include <Wire.h>


Time &Time::get_instance()
{
	static Time time;
	return time;
}


int Time::get_year()
{
	return _year;
}


int Time::get_month()
{
	return _month;
}


const char *Time::get_month_name()
{
	static const char *months[] = {
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December",
	};

	return months[ _month - 1 ];
}


int Time::get_day()
{
	return _day;
}


const char *Time::get_day_suffix()
{
	switch( _day )
	{
	case 1:
	case 21:
	case 31:
		return "st";
	case 2:
	case 22:
		return "nd";
	case 3:
	case 23:
		return "rd";
	default:
		return "th";
	}
}


int Time::get_hours()
{
	return _hours;
}


int Time::get_minutes()
{
	return _minutes;
}


int Time::get_seconds()
{
	return _seconds;
}


void Time::update()
{
	// how many milliseonds have elapsed since we last checked?
	unsigned long millis = ::millis();
	unsigned long delta = millis - _last_millis + _carry;

	// update the previous time and carried-over milliseconds
	_last_millis = millis;
	_carry = delta % 1000;

	// add the seconds that have passed to the time
	_seconds += delta / 1000;
	while( _seconds >= 60 ) {
		_seconds -= 60;
		_minutes++;
		if( _minutes >= 60 ) {
			_minutes -= 60;
			_hours++;
			if( _hours >= 24 ) {
				_hours -= 24;

				// I mean we *could* work out the day... but FUCK IT! We bought
				// some god damn HARDWARE to do that shit!
				probe_rtc();
			}
		}
	}
}


Time::Time()
	:
	_last_millis( millis() ),
	_carry( 0 )
{
	probe_rtc();
}


void Time::probe_rtc()
{
	// get the time from the real-time clock
	int rtc_data[ 7 ];
	RTC.get( rtc_data, true );
	_year = rtc_data[ DS1307_YR ];
	_month = rtc_data[ DS1307_MTH ];
	_day = rtc_data[ DS1307_DAY ];
	_hours = rtc_data[ DS1307_HR ];
	_minutes = rtc_data[ DS1307_MIN ];
	_seconds = rtc_data[ DS1307_SEC ];
}

